import serial
import time
import threading
port = 'COM69'
# 定义常量
MINBAUD = 2400
MAXBAUD = 115200
FOSC = 11059200
MAIN_Fosc = 11059200
FUSER = 24000000
# 假设以下变量已经定义
UartRecvStep = 0
RecvSum = 0
RecvCount = 0
RecvIndex = 0
UartReceived = False
RxBuffer = [0] * 256
timeout = 100*100
# 模拟 HIBYTE 和 LOBYTE 宏
def HIBYTE(num):
    return (num >> 8) & 0xFF

def LOBYTE(num):
    return num & 0xFF


# 串口初始化函数
def serial_init(port, baudrate=MINBAUD):
    try:
        ser = serial.Serial(port, baudrate, timeout=0.001,parity=serial.PARITY_EVEN)
        if ser.is_open:
            print(f"串口 {port} 已打开，波特率: {baudrate}")
            return ser
        else:
            print(f"无法打开串口 {port}")
            return None
    except serial.SerialException as e:
        print(f"串口初始化错误: {e}")
        return None

# 串口发送函数
def serial_send(ser, dat):
    try:
        #print("send:",'{:02x}'.format(dat))
        ser.write(bytes([dat]))
        return True
    except serial.SerialException as e:
        print(f"串口发送错误: {e}")
        return False

# 串口接收函数
def serial_receive(ser):
    try:
        data = ser.read(1)
        if data:
            return data[0]
        else:
            return None
    except serial.SerialException as e:
        print(f"串口接收错误: {e}")
        return None

# 主函数
def main():
      # 请根据实际情况修改串口名称
    ser = serial_init(port, MINBAUD)
    if ser is None:
        return

    # 模拟下载流程
    demo_data = bytes([0x80, 0x00, 0x75, 0xB2, 0xFF, 0x75, 0xB1, 0x00, 0x05, 0xB0, 0x11, 0x0E, 0x80, 0xFA, 0xD8, 0xFE, 0xD9, 0xFC, 0x22])
    if download(ser, demo_data):
        print("下载成功")
    else:
        print("下载失败")

    ser.close()
# 模拟 CommInit 函数
def CommInit():
    global UartRecvStep, RecvSum, RecvCount, RecvIndex, UartReceived,timeout
    UartRecvStep = 0
    RecvSum = 0
    RecvCount = 0
    RecvIndex = 0
    UartReceived = False
    timeout = 100*100
# 波特率计算函数
def BR(n):
    return 65536 - (int)(FOSC / (n) + 2) // 4

def RL(n):
    return 65536 - (int)(FUSER / (n) + 2) // 4

    
# 处理接收到的数据
def handle_received_data(dat):
    global UartRecvStep, RecvSum, RecvCount, RecvIndex, UartReceived, RxBuffer,timeout
    if UartRecvStep == 1:
        if dat != 0xb9:
            CommInit()
        else:
            UartRecvStep += 1
    elif UartRecvStep == 2:
        if dat != 0x68:
            CommInit()
        else:
            UartRecvStep += 1
    elif UartRecvStep == 3:
        if dat != 0x00:
            CommInit()
        else:
            UartRecvStep += 1
    elif UartRecvStep == 4:
        RecvSum = 0x68 + dat
        RecvCount = dat - 6
        RecvIndex = 0
        UartRecvStep += 1
    elif UartRecvStep == 5:
        RecvSum += dat
        RxBuffer[RecvIndex] = dat
        RecvIndex += 1
        if RecvIndex == RecvCount:
            UartRecvStep += 1
    elif UartRecvStep == 6:
        if dat != HIBYTE(RecvSum):
            a = '{:02x}'.format(dat)
            b = '{:02x}'.format(HIBYTE(RecvSum))
            print(f"H校验错误,{a},{b}")
            CommInit()
        else:
            UartRecvStep += 1
    elif UartRecvStep == 7:
        if dat != LOBYTE(RecvSum):
            a = '{:02x}'.format(dat)
            b = '{:02x}'.format(LOBYTE(RecvSum))
            print(f"L校验错误,{a},{b}")
            CommInit()
        else:
            UartRecvStep += 1
    elif UartRecvStep == 8:
        if dat != 0x16:
            CommInit()
        else:
            UartReceived = True
            UartRecvStep += 1
    else:
        CommInit()
        UartRecvStep = 1 if dat == 0x46 else 0
def rcvFun(ser):
    global UartRecvStep, RecvSum, RecvCount, RecvIndex, UartReceived, RxBuffer,timeout
    while True:
        data = ser.read(1)
        if data and not UartReceived:
            dataStr = '{:02x}'.format(data[0])
            #print(f"{UartRecvStep}\{UartReceived} : {dataStr}")
            handle_received_data(data[0])
            print(f"{UartRecvStep}\{UartReceived} : {dataStr}")
            if UartReceived:
                return
def startRcv(ser):
    CommInit()
    # 启动 RX 中断逻辑线程
    rx_thread = threading.Thread(target=rcvFun, args=(ser,))
    rx_thread.daemon = True
    rx_thread.start()
    
# 发送串口通讯数据包
def CommSend(ser, TxBuffer):
    size = len(TxBuffer)
    tempBuffer=[0x46, 0xb9, 0x6A, 0x00]
    sum = 0
    sum = size + 6 + 0x6a
    tempBuffer.append(size + 6)
    for i in range(size):
        tempBuffer.append(TxBuffer[i])
        sum += TxBuffer[i]
    tempBuffer.append(HIBYTE(sum))
    tempBuffer.append(LOBYTE(sum))
    tempBuffer.append(0x16)    
    ser.write(bytes(tempBuffer))
    print("CommSend:",['{:02x}'.format(x) for x in tempBuffer])

# 下载函数
def download(ser, pdat):
    global UartRecvStep, RecvSum, RecvCount, RecvIndex, UartReceived, RxBuffer,timeout
    
    startRcv(ser)
    
    print("waiting")
    while True:
        if UartRecvStep == 0:
            serial_send(ser, 0x7f)
            time.sleep(0.010)

        if UartReceived:
            print(f"{UartRecvStep}\{UartReceived} : {RxBuffer}")
            if len(RxBuffer) > 0 and RxBuffer[0] == 0x50:
                break
            CommInit()
        else:
            time.sleep(0.01)
    
    
    print("设置参数")
    startRcv(ser)
    # 设置参数
    tx_buffer = [0x01, 0x00, 0x00, RL(MAXBAUD) >> 8, RL(MAXBAUD) & 0xff, 0x00, 0x00, 0x97]
    CommSend(ser, tx_buffer)
    
    while True:
        if UartReceived:
            print(f"{UartRecvStep}\{UartReceived} : {RxBuffer}")
            if len(RxBuffer) > 0 and RxBuffer[0] == 0x01:
                break
            
        time.sleep(0.001)
        timeout -= 1
        if timeout == 0:
            return False

    # 准备
    print("准备")
    ser.baudrate = MAXBAUD
    time.sleep(0.01)
    startRcv(ser)
    tx_buffer = [0x05, 0x00, 0x00, 0x5a, 0xa5]
    CommSend(ser, tx_buffer)

    while True:
        if UartReceived:
            print(f"{UartRecvStep}\{UartReceived} : {RxBuffer}")
            if len(RxBuffer) > 0 and RxBuffer[0] == 0x05:
                break
            
        time.sleep(0.001)
        timeout -= 1
        if timeout == 0:
            return False

    # 擦除
    print("擦除")
    time.sleep(0.01)
    startRcv(ser)
    tx_buffer = [0x03, 0x00, 0x00, 0x5a, 0xa5]
    CommSend(ser, tx_buffer)
    timeout = 1000*100

    while True:
        if UartReceived:
            print(f"{UartRecvStep}\{UartReceived} : {RxBuffer}")
            if len(RxBuffer) > 0 and RxBuffer[0] == 0x03:
                break
            CommInit()
        time.sleep(0.001)
        timeout -= 1
        if timeout == 0:
            return False

    # 写用户代码
    print("写用户代码")
    time.sleep(0.01)
    
    addr = 0
    tx_buffer = [0]*256
    tx_buffer[0] = 0x22
    tx_buffer[3] = 0x5a
    tx_buffer[4] = 0xa5


    offset = 5
    while addr < len(pdat):
        if addr < 0x100000:
            tx_buffer[0] |= 0x10
        else:
            tx_buffer[0] &= ~0x10

        tx_buffer[1] = addr >> 8
        tx_buffer[2] = addr & 0xff
        cnt = 0
        while addr < len(pdat):
            tx_buffer[cnt + offset] = pdat[addr]
            addr += 1
            cnt += 1
            if cnt >= 128:
                break
        startRcv(ser)
        CommSend(ser, tx_buffer[:cnt + offset])

        while True:
            if UartReceived:
                print(f"{UartRecvStep}\{UartReceived} : {RxBuffer}")
                if len(RxBuffer) > 2 and RxBuffer[0] == 0x02 and RxBuffer[1] == ord('T'):
                    break
                return False
            time.sleep(0.001)    
            timeout -= 1
            if timeout == 0:
                return False

        tx_buffer[0] = 0x02

    return True

if __name__ == "__main__":
    main()    